@@ -18,5 +18,5 @@ notempty1=';comment=val'
 empty=''
 python_interpolate='%(dup1)s/blah'
 interpolate2='${dup1}/blah'
-caps='not significant'
+Caps='not significant'
 combine=sections
