@@ -1,14 +1,18 @@
 #!/bin/bash
 
 trap "exit 130" INT
-cleanup() { rm -f err noequals*.ini test.ini ltest.ini good.ini example.ini; exit; }
-trap cleanup EXIT
+cleanup() { rm -f err noequals*.ini test.ini ltest.ini good.ini example.ini; exit "$1"; }
+trap 'cleanup $?' EXIT
 
-export PATH=..:$PATH
+crudini() { ../crudini.py "$@"; }
 
 test=0
 
-fail() { test=$(($test+1)); echo "Test $test FAIL (line ${BASH_LINENO[0]})"; exit 1; }
+fail() {
+  test=$(($test+1))
+  printf "Test $test \033[1;31mFAIL\033[m (line ${BASH_LINENO[0]})\n"
+  exit 1
+}
 ok() { test=$(($test+1)); echo "Test $test OK (line ${BASH_LINENO[0]})"; }
 
 cp ../example.ini .
@@ -113,6 +117,9 @@ for mode in '' '--inplace'; do
   # Maintain space separation
   crudini $mode --set example.ini section1 nospace val
   grep -q '^nospace=val' example.ini && ok || fail
+
+  crudini $mode --ini-options= --set example.ini section1 nospace val
+  grep -q '^nospace=val' example.ini && ok || fail
 done
 
 # value is optional
@@ -450,11 +457,24 @@ crudini --set example.ini list list2 'v1,v2,v3' || fail
 #  explicit
 crudini --list --list-sep=, --del example.ini list list2 v3 || fail
 test "$(crudini --get example.ini list list2)" = 'v1,v2' && ok || fail
+#  whitespace separated (while maintaining newline separation)
+crudini --list --list-sep= --set example.ini list list3 v2 || fail # ignored
+crudini --list --list-sep= --set example.ini list list3 v3 || fail
+test "$(crudini --get example.ini list list3)" = $'\nv1\nv2\nv3' && ok || fail
 
 # Delete honoring --existing
 crudini --list --existing --del example.ini nolist list1 v3 2>/dev/null && fail || ok
 crudini --list --existing --del example.ini list nolist1 v3 2>/dev/null && fail || ok
 
+# Support clearing a list by not specifing a value
+printf '%s\n' 'empty' 'nonempty = 1' > test.ini
+crudini --list --set test.ini '' nonempty 2>/dev/null && ok || fail
+test "$(crudini --get test.ini '' nonempty 2>&1)" = '' && ok || fail
+crudini --list --set test.ini '' empty 'v1' 2>/dev/null && ok || fail
+test "$(crudini --get test.ini '' empty 2>&1)" = 'v1' && ok || fail
+
+# --------------------------------------------------------
+
 # support parsing from stdin
 test "$(printf '%s\n' global=1 | crudini --get - '' global)" = 1 && ok || fail
 
@@ -500,6 +520,46 @@ done
 crudini --set noequals.ini noequals new_equals '' || fail
 diff -u noequals.ini noequals_new.ini && ok || fail
 
+# Test updating of single token parameters
+#  Ensure no delimitier added with unspecified value
+test "$(crudini --output=- --set noequals.ini noequals param1 \
+        | grep param1)" = 'param1' && ok || fail
+#  Ensure delimitier added with unspecified value with --list
+test "$(crudini --list --output=- --set noequals.ini noequals param1 \
+        | grep param1)" = 'param1 = ' && ok || fail
+#  Ensure delimitier added with empty value
+test "$(crudini --output=- --set noequals.ini noequals param1 '' \
+        | grep param1)" = 'param1 = ' && ok || fail
+#  Ensure correct spacing with --ini-options=nospace
+test "$(crudini --ini-options=nospace --output=- --set noequals.ini \
+        noequals param1 value1 \
+        | grep param1)" = 'param1=value1' && ok || fail
+
 # Test can read windows format files
-printf '%s\r\n' '' 'param = value' > test.ini
-crudini --get test.ini DEFAULT param > /dev/null || fail
+printf '%s\r\n' '' 'empty' 'param=value' > test.ini
+test "$(crudini --get test.ini DEFAULT param)" = 'value' && ok || fail
+# Test can maintain windows format files
+diff -u <(crudini --output=- --ini-options=nospace \
+          --set test.ini '' param value) \
+        test.ini && ok || fail
+printf '%s\r\n' '' 'empty=' > test.ini
+diff -u <(crudini --output=- --ini-options=nospace \
+          --set test.ini '' empty '') \
+        test.ini && ok || fail
+printf '%s\r\n' '' 'empty' > test.ini
+diff -u <(crudini --output=- --ini-options=nospace \
+          --set test.ini '' empty) \
+        test.ini && ok || fail
+
+# Test closed stdin
+(0<&- crudini --help >/dev/null) && ok || fail
+
+# Test closed stdout
+(>&- crudini --get test.ini section param value 2>/dev/null) && fail || ok
+(>&- crudini --set - section param value 2>/dev/null) && fail || ok
+(>&- crudini --set test.ini section param value) && ok || fail
+
+# Test --ini-options=nospace
+diff -u <(crudini --output=- --ini-options=nospace \
+          --set nospace-in.ini '' new val) \
+        nospace-out.ini && ok || fail
